// BossPuzzle.java                                  MM 2013
import Tools.IO.*;                              // Eingaben

/**
 * Boss-Puzzle, Spiel fuer eine Person.
 * Der Spieler ordnet durch Verschieben
 * die Buchstaben von A bis O.
 * @author Monika Meiler
 */
public class BossPuzzle
{
/* ----------------------------------------------------- */
                                               // Attribute
/**
 * Puzzle.
 */
  private char[][] puzzle;

/**
 * Leerfeldzeile.
 */
  private int leerX;

/**
 * Leerfeldspalte.
 */
  private int leerY;

/**
 * Spielrundenzaehler.
 */
  private int runde;

/**
 * Spielfeldbreite, Original 4.
 */
  private int breite = 4;

/* ----------------------------------------------------- */
                                             // Spielaufbau
/**
 * Spiel starten.
 */
  public void starten()
  {
// Erlaeuterung
    System.out.println();
    erlaeuterung();

    System.out.println();
    IOTools.readLine( "Weiter (ENTER) ");

// Spielstart
    char weiter = 'j';
    do
    {
      System.out.println();                        // Spiel
      spielen();

      System.out.println();                   // Auswertung
      auswertung();
                                                  // Weiter
      weiter = IOTools.readChar( "Noch einmal (j/n)? ");
    } while( weiter == 'j');

// Beenden
    System.out.println();
    System.out.println( "Spiel beendet!");
  }

/* ----------------------------------------------------- */
                                                   // Spiel
/**
 * Spielerlaeuterung.
 */
  private void erlaeuterung()
  {
    String str = "Boss-Puzzle, ";
    str += "Spiel fuer eine Person.\n\n";

    str += "Der Spieler ordnet durch Verschieben ";
    str += "die Buchstaben von A bis O.";

    System.out.println( str);
  }

/**
 * Spielen.
 */
  private void spielen()
  {
// Puzzle
    erzeugePuzzle();

// Anzeigen
    zeigePuzzle();

// Spielen
    runde = 0;
    do
    {
      char eintrag = IOTools.readChar( "Eingabe: ");

      if( ziehe( eintrag))              // Feld verschieben
      {
        runde++;
        zeigePuzzle();                           // Anzeige
      }
      else
        System.out.println( "Fehlerhafte Eingabe");
    } while( !fertig());
  }

/**
 * Spielauswertung.
 */
  private void auswertung()
  {
    String str = "Runde: " + runde + ", ";
    str += "Du hast das Puzzle geloest.";
    System.out.println( str);
  }

/* ----------------------------------------------------- */
                                                  // Puzzle
/**
 * Erzeugt Puzzle.
 */
  private void erzeugePuzzle()
  {
// Puzzle
    puzzle = new char[ breite][ breite];
    char eintrag = 'A';
    for( int i = 0; i < breite; i++)
    {
      for( int j = 0; j < breite; j++)
      {
        puzzle[ i][ j] = eintrag;
        eintrag++;
      }
    }
// Leerfeld
    leerX = leerY = breite - 1;
    puzzle[ leerX][ leerY] = ' ';

// Mischen durch Verschieben
    mischePuzzle( 100 * breite);
  }

/**
 * Mische Puzzle, durch zufaelliges Verschieben.
 * @param mischTiefe Mindestzahl der Verschiebungen
 */
  private void mischePuzzle( int mischTiefe)
  {
// Wuerfel, max. 4 Nachbarfelder
    Wuerfel wuerfel = new Wuerfel();
    wuerfel.setWuerfel( 0, 3);
                                              //     1
                                              //  2 leer 0
                                              //     3

    for( int i = 0; i < mischTiefe; i++)
    {                                // Tausche mit Nachbar
      int tausch = wuerfel.wuerfeln();
      int x = leerX, y = leerY;
      switch( tausch)
      {
        case 0: y = y + 1; break;
        case 1: x = x - 1; break;
        case 2: y = y - 1; break;
        case 3: x = x + 1;
      }
      if( !ziehe( x, y)) i--;                // noch einmal
    }

// Leerfeld unten rechts anordnen
    int x = leerX, y = leerY;
                                  // nach unten verschieben
    for( ; x < breite - 1; x++) ziehe( x + 1, y);
                                 // nach rechts verschieben
    for( ; y < breite - 1; y++) ziehe( x, y + 1);
  }

/**
 * Puzzle als String.
 */
  private void zeigePuzzle()
  {
    String str = "";
    for( int i = 0; i < breite; i++)
    {
      str+= " | ";                            // neue Zeile
      for( int j = 0; j < breite; j++)
        str += puzzle[ i][ j] + " | ";
      str += "\n";                        // Zeilenvorschub
    }
    System.out.println( str);
  }

/* ----------------------------------------------------- */
                                                     // Zug
/**
 * Zug von Feld( x, y ) nach Leerfeld.
 * @param x Zeile
 * @param y Spalte
 * @return true, falls gezogen
 */
  private boolean ziehe( int x, int y)
  {
// Feld( x, y) liegt nicht im Puzzle
    if( x < 0 || x >= breite || y < 0 || y >= breite)
      return false;

// Feld ist Nachbar des Leerfeldes
    if( x == leerX && ( y == leerY-1 || y == leerY+1)
     || y == leerY && ( x == leerX-1 || x == leerX+1))
    {
                                         // Verschiebe Feld
      puzzle[ leerX][ leerY] = puzzle[ x][ y];

      leerX = x;                          // neues Leerfeld
      leerY = y;
      puzzle[ leerX][ leerY] = ' ';

      return true;                               // gezogen
    }
    return false;                          // nicht gezogen
  }

/**
 * Zug von Feld mit Eintrag nach Leerfeld.
 * @param eintrag Feldeintrag
 * @return true, falls gezogen
 */
  private boolean ziehe( char eintrag)
  {
// Buchstabe kommt nicht vor
    if( eintrag < 'A' ||
        eintrag > ('A' + breite * breite - 2))
        return false;

// Feldindex
    int x = 0, y = 0;
    for( int i = 0; i < breite; i++)
    {
      for( int j = 0; j < breite; j++)
        if( puzzle[ i][ j] == eintrag)
        {
          x = i;
          y = j;
          break;
        }
    }
// ziehen
    return ziehe( x, y);
  }

/**
 * Puzzle in Ausgangsstellung.
 */
  public boolean fertig()
  {
// Leerfeld
    if( leerX != breite-1 || leerY != breite-1)
      return false;
// Eintraege
    char eintrag = 'A';
    for( int i = 0; i < breite; i++)
    {
      for( int j = 0; j < breite; j++)
      {                                         // Leerfeld
        if( i == breite-1 && j == breite-1) return true;
                                                 // Eintrag
        if( puzzle[ i][ j] != eintrag) return false;
        eintrag++;
      }
    }
    return false;                      // wird nie erreicht
  }

/* ----------------------------------------------------- */
                                                // Programm
/**
 * Programm, erzeugt und startet das Bosspuzzle.
 */
  public static void main( String[] args)
  {
// Puzzlespiel erzeugen
    BossPuzzle boss = new BossPuzzle();

// Puzzlespiel starten
    boss.starten();
  }
}
