// Nimm.java                                        MM 2013
import Tools.IO.*;                              // Eingaben

/**
 * Nimmspiel, Spiel gegen den Computer
 * oder mit mindestens zwei Spielern.
 * Die Spieler nehmen abwechseld 1 bis 3 Steine 
 * von einem Steinhaufen. Verloren hat der Spieler,
 * der den letzten Stein nehmen muss.
 * @author Monika Meiler
 */
public class Nimm
{
/* ----------------------------------------------------- */
                                               // Attribute
/**
 * Alle Spieler.
 */
  private Spieler[] alleSpieler;

/**
 * Letzter Spieler.
 */
  private Spieler dran;

/**
 * Steine im Spiel, klassisch 21.
 */
  private int steine = 21;

/**
 * Spielrundenzaehler.
 */
  private int runde = 0;

/* ----------------------------------------------------- */
                                             // Spielaufbau
/**
 * Spiel starten.
 */
  public void starten()
  {
// Erlaeuterung
    System.out.println();
    erlaeuterung();

    System.out.println();
    IOTools.readLine( "Weiter (ENTER) ");

// Spielstart
    char weiter = 'j';
    do
    {
      System.out.println();                        // Spiel
      spielen();

      System.out.println();                   // Auswertung
      auswertung();

                                                  // Weiter
      weiter = IOTools.readChar( "Noch einmal (j/n)? ");
    } while( weiter == 'j');

// Beenden
    System.out.println();
    System.out.println( "Spiel beendet!");
  }

/* ----------------------------------------------------- */
                                                   // Spiel
/**
 * Spielerlaeuterung.
 */
  private void erlaeuterung()
  {
    String str = "Nimmspiel, ";
    str += "gegen den Computer ";
    str += "oder mit mindestens zwei Spielern.\n\n";

    str += "Die Spieler nehmen abwechseld 1 bis 3 ";
    str += "Steine von einem Steinhaufen.\n";
    str += "Verloren hat der Spieler, ";
    str += "der den letzten Stein nehmen muss.";

    System.out.println( str);
  }

/**
 * Eingabe der Steinanzahl und der Spieler, Spielen.
 */
  private void spielen()
  {
// Steinanzahl
    Wuerfel wuerfel = new Wuerfel();
    wuerfel.setWuerfel( 10, 30);
    steine = wuerfel.wuerfeln();

// Liest Spieler ein
    int anzahlSpieler = eingabeSpieler();

// Spiel
    if( anzahlSpieler > 1) spielerToSpieler();
    if( anzahlSpieler == 1) computerToSpieler();
  }

/**
 * Spielauswertung.
 * Der letzte Spieler ist der Verlierer.
 */
  private void auswertung()
  {
    String str = "Runde: " + runde + ", ";
    str += dran.getName() + ", Du hast verloren! ";
    str += "Hast Du die Strategie erkannt?";
    System.out.println( str);
  }

/* ----------------------------------------------------- */
                                            // Alle Spieler
/**
 * Liest alle Namen der Spieler ein.
 * @return Anzahl der eingegebenen Spieler
 */
  private int eingabeSpieler()
  {
// Spieleranzahl
    System.out.println();
    int anzahlSpieler = IOTools.readInteger
    ( "Wie viel Spieler spielen mit (mindstens 1)? ");

// Spielerfeld
    if( anzahlSpieler == 1)      // Computer ist 2. Spieler
    {
      alleSpieler = new Spieler[ 2];
      Spieler computer = new Spieler( "Computer");
      alleSpieler[ 1] = computer;
    }

    if( anzahlSpieler > 1)
      alleSpieler = new Spieler[ anzahlSpieler];

    if( anzahlSpieler > 0)               // Eintrag Spieler
    {
      for( int i = 0; i < anzahlSpieler; i++)
      {
        String name = IOTools.readLine
        ( "Name des " + (i + 1) + ". Spielers: ");

        Spieler spieler = new Spieler( name);
        alleSpieler[ i] = spieler;
      }
    }

    return anzahlSpieler;
  }

/* ----------------------------------------------------- */
                                   // Spieler gegen Spieler
/**
 * Spieler spielen gegeneinander.
 */
  private void spielerToSpieler()
  {
// Nehmen
    runde = 0;
    do
    {
      System.out.println();
      if( steine == 1)
        System.out.println( "Nur noch ein Stein!");
      else
        System.out.println
        ( "Es sind noch " + steine + " Steine da!");

      dran = alleSpieler[ runde % alleSpieler.length];

      int nehme = spielerNehmen();
      steine -= nehme;
      runde++;
    } while( steine > 0);
  }

/* ----------------------------------------------------- */
                                  // Spieler gegen Computer
/**
 * Spieler spielt gegen den Computer.
 */
  private void computerToSpieler()
  {
// Nehmen
    runde = 0;
    do
    {
      System.out.println();
      if( steine == 1)
        System.out.println( "Nur noch ein Stein!");
      else
        System.out.println
        ( "Es sind noch " + steine + " Steine da!");

      int i = runde % 2;
      dran = alleSpieler[ i];

      int nehme;
      if( i == 1) nehme = computerNehmen();
      else nehme = spielerNehmen();

      steine -= nehme;
      runde++;
    } while( steine > 0);
  }

/* ----------------------------------------------------- */
                                                  // Nehmen
/**
 * Spieler nimmt 1 bis 3 Steine.
 * @return Anzahl der genommenen Steine
 */
  private int spielerNehmen()
  {
    int nehme;
    do
    {
      System.out.println
      ( dran.getName() + " nimmt!");

      nehme = IOTools.readInteger
              ( "Wie viele Steine (1,2,3) ");
    } while( nehme < 1 || nehme > 3 || nehme > steine);
    
    return nehme;
  }


/**
 * Computer nimmt 1 bis 3 Steine, mit Strategie.
 * @return Anzahl der genommenen Steine
 */
  private int computerNehmen()
  {
    int nehme = (steine + 3) % 4;
    if( nehme == 0) nehme = 1;         // Unguenstiger Fall
    System.out.println
    ( "Der Computer nimmt " + nehme + ".");

    return nehme;
  }

/* ----------------------------------------------------- */
                                                // Programm
/**
 * Programm, erzeugt und startet das Nimmspiel.
 */
  public static void main( String[] args)
  {
// Spiel erzeugen
    Nimm nimm = new Nimm();

// Spiel starten
    nimm.starten();
  }
}
